package com.ether;

import java.util.List;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.transaction.annotation.Transactional;

import com.ether.dao.instrument.InstrumentDAO;
import com.ether.instrument.Instrument;
import com.ether.instrument.InstrumentType;

/**
 * @author vmipsl
 * @date 24 jan 2011 
 */
public class InstrumentServiceImpl implements InstrumentService
{
    @NotNull
    @Transactional(readOnly = true)
    public Integer getNbInstruments() throws ServiceException
    {
	try
	{
	    return _instrumentDAO.getNbInstruments();
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.INSTRUMENT_NOT_FOUND, e);
	}
    }

    @Nullable
    @Transactional(readOnly = true)
    public List<Instrument> getAllInstruments() throws ServiceException
    {
	try
	{
	    return _instrumentDAO.selectAll();
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.INSTRUMENT_NOT_FOUND, e);
	}
    }

    @Nullable
    @Transactional(readOnly = true)
    public Instrument getInstrumentById(@NotNull final Long instrumentId) throws ServiceException
    {
	try
	{
	    return _instrumentDAO.selectByPrimaryKey(instrumentId);
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.INSTRUMENT_NOT_FOUND, e);
	}
    }
    
    @NotNull
    @Transactional(rollbackFor = ServiceException.class)
    public Long createInstrument(@NotNull final Instrument instrument) throws ServiceException
    {
	try
	{
	    return _instrumentDAO.insert(instrument);
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.PERSISTENCE, e);
	}
    }

    @Transactional(rollbackFor = ServiceException.class)
    public void removeInstrument(@NotNull final Long instrumentId) throws ServiceException
    {
	try
	{
	    _instrumentDAO.deleteByPrimaryKey(instrumentId);
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.PERSISTENCE, e);
	}
    }
    
    @Nullable
    @Transactional(readOnly = true)
    public List<Instrument> getInstrumentsByStationId(@NotNull final Long stationId) throws ServiceException
    {
	try
	{
	    return _instrumentDAO.getInstrumentsByStationId(stationId);
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.INSTRUMENT_BY_STATION_NOT_FOUND, e);
	}
    }

    @Nullable
    @Transactional(readOnly = true)
    public List<Instrument> getInstrumentsByType(@NotNull final InstrumentType type) throws ServiceException
    {
	try
	{
	    return _instrumentDAO.getInstrumentsByType(type);
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.INSTRUMENT_BY_STATION_NOT_FOUND, e);
	}
    }
    
    @Required
    public void setInstrumentDAO(final InstrumentDAO instrumentDAO)
    {
    	_instrumentDAO = instrumentDAO;
    }

    private InstrumentDAO _instrumentDAO;
}
