package com.ether;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.transaction.annotation.Transactional;

import com.ether.dao.plot.PlotDAO;

/**
 * @author rboipsl
 * @date 23 feb 2011 
 */
public class PlotServiceImpl implements PlotService
{
        
	@Nullable
	@Transactional(readOnly = true)
	public List<String> getPlotsByDateByInstrumentByStation(@NotNull final String instrument, @NotNull String pdate, @NotNull String ft, @NotNull String station) 
		throws ServiceException
	{
		try {
			
			final DateFormat pFormatDate = new SimpleDateFormat(ft);
			String fdate;
			final String tt="yyyyMMdd";
			if (ft.compareTo(tt) == 0) fdate="19700101";
			else fdate = "1970";
			
			if (pdate.compareTo(fdate) == 0 ) {
				final Date ldate = _plotDAO.getLastDateByInstrumentByStation(instrument,station);		
				
				//ldate="20070101";
				//ldate="20060101";
				
				String s1="-", s2="", d1="01";
				fdate=ldate.toString().replaceAll(s1,s2);
				
				s1=" 00:00:00.0";
				fdate=fdate.replaceAll(s1,s2);
				fdate=fdate.substring(0,6);
				
				fdate=fdate.concat(d1);
				pdate=fdate;
			}
			else {
				fdate=pdate;
			}			
			
			final Date pFormatDate_deb = pFormatDate.parse(fdate);
			Date pFormatDate_fin;
			if (ft.compareTo(tt) == 0) pFormatDate_fin = DateUtils.datesAddMonth(pFormatDate_deb,1);
			else pFormatDate_fin = DateUtils.datesAddYear(pFormatDate_deb,1);
			
			return _plotDAO.getPlotsByDateByInstrumentByStation(instrument, pFormatDate_deb, pFormatDate_fin, station);			

		} catch (ParseException e) {
			throw new ServiceException(ServiceException.ServiceCode.DATES_PARSE_ERROR, e);
		} catch (PersistenceException e) {
			throw new ServiceException(ServiceException.ServiceCode.INSTRUMENT_ERROR, e);
		}
	}
	
	@Transactional(readOnly = true)	
	public List<String> getPlotsByDateByInstrument(@NotNull final String instrument, @NotNull String pdate, @NotNull String ft) 
		throws ServiceException
	{
		try {
		
			DateFormat pFormatDate = new SimpleDateFormat(ft);
			String fdate;
			String tt="yyyyMMdd";
			if (ft.compareTo(tt) == 0) fdate="19700101";
			else fdate = "1970";
			
			if (pdate.compareTo(fdate) == 0 ) {
				List<Date> ldates = _plotDAO.getDatesByInstrument(instrument);
			
				String ldate=ldates.get(0).toString();
			
				String s1="-", s2="", d1="01";
				fdate=ldate.replaceAll(s1, s2);
				s1=" 00:00:00.0";
				fdate=fdate.replaceAll(s1, s2);
				fdate=fdate.substring(0, 6);
			
			
				fdate=fdate.concat(d1);
				pdate=fdate;
			
			}
			else {
				fdate=pdate;
			}			
		
			Date pFormatDate_deb = pFormatDate.parse(fdate);
			Date pFormatDate_fin;
			if (ft.compareTo(tt) == 0) pFormatDate_fin = DateUtils.datesAddMonth(pFormatDate_deb,1);
			else pFormatDate_fin = DateUtils.datesAddYear(pFormatDate_deb,1);
		
			return _plotDAO.getPlotsByDateByInstrument(instrument, pFormatDate_deb, pFormatDate_fin);			
		
		} catch (ParseException e) {
			throw new ServiceException(ServiceException.ServiceCode.DATES_PARSE_ERROR, e);
		} catch (PersistenceException e) {
			throw new ServiceException(ServiceException.ServiceCode.INSTRUMENT_ERROR, e);
		}
	}

	/**
	 * 
	 * @return list of 2 dates :
	 * 		- first date corresponding to begin date
	 * 		- second date corresponding to end date
	 * @throws PersistenceException 
	 * @throws ParseException 
	 */
	private List<Date> getFormatDates(@NotNull final String instrument, @NotNull String pdate, @NotNull final String formatDate) 
		throws PersistenceException, ParseException
	{
		DateFormat pFormatDate = new SimpleDateFormat(formatDate);
		String fdate;
		String tt="yyyyMMdd";
		if (formatDate.compareTo(tt) == 0) fdate="19700101";
		else fdate = "1970";
		
		if (pdate.compareTo(fdate) == 0 ) {
			List<Date> ldates = _plotDAO.getDatesByInstrument(instrument);
			
			String ldate=ldates.get(0).toString();				
			
			String s1="-", s2="", d1="01";
			fdate=ldate.replaceAll(s1, s2);
			s1=" 00:00:00.0";
			fdate=fdate.replaceAll(s1, s2);
			fdate=fdate.substring(0, 6);
			
			fdate=fdate.concat(d1);
			pdate=fdate;
			
		} else {
			fdate=pdate;
		}			
		
		Date pFormatDate_deb = pFormatDate.parse(fdate);
		Date pFormatDate_fin;
		if (formatDate.compareTo(tt) == 0) pFormatDate_fin = DateUtils.datesAddMonth(pFormatDate_deb,1);
		else pFormatDate_fin = DateUtils.datesAddYear(pFormatDate_deb,1);
		
		final List<Date> resultDates = new ArrayList<Date>();
		resultDates.set(0, pFormatDate_deb);
		resultDates.set(1, pFormatDate_fin);
		return resultDates;
	}
	
	@Required
    public void setPlotDAO(final PlotDAO plotDAO)
    {
		_plotDAO = plotDAO;
    }

    private PlotDAO _plotDAO;
}
