package com.ether;

import java.util.List;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.transaction.annotation.Transactional;

import com.ether.dao.station.StationDAO;
import com.ether.station.Station;

/**
 * @author vmipsl
 * @date 24 jan 2011
 */
public class StationServiceImpl implements StationService
{
    @NotNull
    @Transactional(readOnly = true)
    public Integer getNbStations() throws ServiceException
    {
	try
	{
	    return _stationDAO.getNbStations();
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.STATION_NOT_FOUND, e);
	}
    }

    @Nullable
    @Transactional(readOnly = true)
    public List<Station> getAllStations() throws ServiceException
    {
	try
	{
	    return _stationDAO.selectAll();
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.STATION_NOT_FOUND, e);
	}
    }

    @Nullable
    @Transactional(readOnly = true)
    public Station getStationById(@NotNull final Long stationId) throws ServiceException
    {
	try
	{
	    return _stationDAO.selectByPrimaryKey(stationId);
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.STATION_NOT_FOUND, e);
	}
    }

    @Nullable
    @Transactional(readOnly = true)
    public Station getStationByName(@NotNull final String stationName) throws ServiceException
    {
	try
	{
	    return _stationDAO.getStationByName(stationName);
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.STATION_NOT_FOUND, e);
	}
    }

    
    @NotNull
    @Transactional(rollbackFor = ServiceException.class)
    public Long createStation(@NotNull final Station station)
	    throws ServiceException
    {
	try
	{
	    return _stationDAO.insert(station);
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.PERSISTENCE, e);
	}
    }

    @Transactional(rollbackFor = ServiceException.class)
    public void removeStation(@NotNull final Long stationId)
	    throws ServiceException
    {
	try
	{
	    _stationDAO.deleteByPrimaryKey(stationId);
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.PERSISTENCE, e);
	}
    }

    @Nullable
    @Transactional(readOnly = true) 
    public List<Station> getStationsByAltitude(@NotNull final String altitude) 
    	throws ServiceException
    {
	try
	{
	    return _stationDAO.getStationsByAltitude(altitude);
	}
	catch (PersistenceException e)
	{
	    throw new ServiceException(ServiceException.ServiceCode.STATION_BY_ALTITUDE_NOT_FOUND, e);
	}
    }
    
    /**
     * Setter to allows Spring to inject stationDAO implementation
     * @param stationDAO : object (implementation of StationDAO interface) to inject.
     */
    @Required
    public void setStationDAO(final StationDAO stationDAO)
    {
    	_stationDAO = stationDAO;
    }

    private StationDAO _stationDAO;
}
