package com.ether;

import static org.junit.Assert.assertNotNull;

import java.util.List;

import org.jetbrains.annotations.Nullable;
import org.junit.Assert;
import org.junit.Test;

import com.ether.instrument.Instrument;
import com.ether.instrument.InstrumentType;
import com.ether.station.Station;

public class InstrumentTest extends ServiceTestUtils<InstrumentService>
{
    public InstrumentTest() {
	super("instrumentService");
    }

    @Test
    public void testInstrumentFields() throws Exception
    {
	// Create station
	final Station station = createStation(STATION_NAME, STATION_ALTITUDE, STATION_LATITUDE, STATION_LONGITUDE);
	assertNotNull(station);
	
	final Instrument instrument = new Instrument();
	instrument.setName(INSTRUMENT_NAME);
	instrument.setStation(station);
	instrument.setType(InstrumentType.LIDAR);
	final Long instrumentId = getService().createInstrument(instrument);
	assertNotNull(instrumentId);

	final Instrument extractedInstrument = getService().getInstrumentById(instrumentId);
	assertNotNull(extractedInstrument);

	Assert.assertEquals(instrumentId, extractedInstrument.getId());
	Assert.assertEquals(INSTRUMENT_NAME, extractedInstrument.getName());
	Assert.assertEquals(InstrumentType.LIDAR, extractedInstrument.getType());
	Assert.assertEquals(station.getId(), extractedInstrument.getStation().getId());
    }

    @Test
    public void testInstrumentSelectByStationOrType() throws Exception
    {
	// Create station1
	final Station station1 = createStation(STATION_NAME+"1", STATION_ALTITUDE, STATION_LATITUDE, STATION_LONGITUDE);
	assertNotNull(station1);

	// Create station2
	final Station station2 = createStation(STATION_NAME+"2", STATION_ALTITUDE, STATION_LATITUDE, STATION_LONGITUDE);
	assertNotNull(station2);

	// Create some instruments for station1
	final Instrument instrument1A = createInstrument(INSTRUMENT_NAME+"1A", station1, InstrumentType.LIDAR);
	assertNotNull(instrument1A);
	final Instrument instrument1B = createInstrument(INSTRUMENT_NAME+"1B", station1, InstrumentType.DOBSON);
	assertNotNull(instrument1B);

	// Create some instruments for station2
	final Instrument instrument2A = createInstrument(INSTRUMENT_NAME+"2A", station2, InstrumentType.OCC);
	assertNotNull(instrument2A);
	final Instrument instrument2B = createInstrument(INSTRUMENT_NAME+"2B", station2, InstrumentType.DOBSON);
	assertNotNull(instrument2B);
	final Instrument instrument2C = createInstrument(INSTRUMENT_NAME+"2C", station2, InstrumentType.SAOZ);
	assertNotNull(instrument2C);
	
	// Select all instruments
	final List<Instrument> instruments = getService().getAllInstruments();
	assertNotNull(instruments);
	Assert.assertEquals(instruments.size(), 5);
	
	// Select by station
	final List<Instrument> instrumentsStation1 = getService().getInstrumentsByStationId(station1.getId());
	assertNotNull(instrumentsStation1);
	final List<Instrument> instrumentsStation2 = getService().getInstrumentsByStationId(station2.getId());
	assertNotNull(instrumentsStation2);
	Assert.assertEquals(instrumentsStation1.size(), 2);
	Assert.assertEquals(instrumentsStation2.size(), 3);

	// Select by type
	final List<Instrument> instrumentsType = getService().getInstrumentsByType(InstrumentType.DOBSON);
	assertNotNull(instrumentsType);
	Assert.assertEquals(instrumentsStation1.size(), 2);
	Assert.assertEquals(instrumentsType.get(0).getId(), instrument1B.getId());
	Assert.assertEquals(instrumentsType.get(1).getId(), instrument2B.getId());	
    }

    private Instrument createInstrument(@Nullable final String instrumentName, @Nullable final Station station, @Nullable final InstrumentType type)
	throws ServiceException
    {
	final Instrument instrument = new Instrument();
	instrument.setName(instrumentName);
	instrument.setStation(station);
	instrument.setType(type);
	final Long instrumentId = getService().createInstrument(instrument);
	assertNotNull(instrumentId);
	
	instrument.setId(instrumentId);
	return instrument;
    }

    @Override
    protected void beforeServiceTest() throws Exception
    {
	afterServiceTest();
    }

    private static final String INSTRUMENT_NAME = "InstrumentNameTest";

    private static final String STATION_NAME = "stationNameTest";
    private static final String STATION_ALTITUDE = "10";
    private static final String STATION_LATITUDE = "20";
    private static final String STATION_LONGITUDE = "45";
    
}
