package com.ether;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.Nullable;
import org.springframework.transaction.annotation.Transactional;

import com.ether.instrument.Instrument;
import com.ether.station.Station;

public abstract class ServiceTestUtils<SERVICE extends Service> 
	extends	HibernateTestUtils<SERVICE>
{
    protected ServiceTestUtils(final String serviceBeanName) {
	super(null, serviceBeanName, getAllDependencies(), DEFAULT_SESSION_FACTORY_BEAN_NAME);
    }

    protected void deleteAllKnownData() throws Exception
    {
	deleteAllDatas( getAllKnownDomainClasses() );
    }

    public static String[] getAllDependencies()
    {
	return new String[] {
		"classpath:dao-context.xml",
		"classpath:service-context.xml",
		"classpath:hibernate_test.cfg.xml",		
	};
    }

    @SuppressWarnings("rawtypes")
    public static Class[] getAllKnownDomainClasses()
    {
	return new Class[] { 
		Station.class, 
		Instrument.class, };
    }

    /**
     * Executes operations before running test.
     */
    protected abstract void beforeServiceTest() throws Exception;

    /**
     * Executes operations after running test.
     */
    protected final void afterServiceTest() throws Exception
    {
	 //deleteAllKnownData();	
    }

    protected final void beforeHibernateTest() throws Exception
    {
	beforeServiceTest();
    }

    protected final void afterHibernateTest() throws Exception
    {
	afterServiceTest();
    }

    @Transactional
    public Station createStation(@Nullable final String stationName, @Nullable final String altitude, @Nullable final String latitude, @Nullable final String longitude)
    	throws ServiceException
    {
	final Station station = new Station();
	station.setName(stationName);
	station.setAltitude(altitude);
	station.setLatitude(latitude);
	station.setLongitude(longitude);
	
	final Long stationId = getStationService().createStation(station);
	station.setId(stationId);
	return station;
    }

    protected final SERVICE getService()
    {
	return getBean();
    }

    protected StationService getStationService()
    {
        return (StationService) getApplicationContext().getBean( "stationService" );
    }

    private static final Log LOGGER = LogFactory.getLog(ServiceTestUtils.class);
    private static final String DEFAULT_SESSION_FACTORY_BEAN_NAME = "sessionFactory";
}
