package com.ether;

import static org.junit.Assert.assertNotNull;

import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import com.ether.station.Station;

public class StationTest extends ServiceTestUtils<StationService>
{
    public StationTest() {
	super("stationService");
    }

    @Test
    public void testStationFields() throws Exception
    {
	final Station station = createStation(STATION_NAME, STATION_ALTITUDE, STATION_LATITUDE, STATION_LONGITUDE);
	assertNotNull(station);

	final Station extractedStation = getService().getStationById(station.getId());
	Assert.assertEquals(station.getId(), extractedStation.getId());
	Assert.assertEquals(station.getName(), extractedStation.getName());
	Assert.assertEquals(station.getAltitude(), extractedStation.getAltitude());
	Assert.assertEquals(station.getLatitude(), extractedStation.getLatitude());
	Assert.assertEquals(station.getLongitude(), extractedStation.getLongitude());
    }

    @Test
    public void testCreateAndRemoveStation() throws Exception
    {
	final Integer sizeBeforeCreate = getService().getNbStations();
	assertNotNull(sizeBeforeCreate);

	// Create station
	final Station station = new Station();
	station.setName("stationTest");
	final Long stationId = getService().createStation(station);
	assertNotNull(stationId);

	// Verify number of stations in DB after creation
	final Integer sizeAfterCreate = getService().getNbStations();
	assertNotNull(sizeAfterCreate);
	Assert.assertEquals((Integer) (sizeBeforeCreate + 1), sizeAfterCreate);

	// Verify getAllStations
	final List<Station> stations = getService().getAllStations();
	assertNotNull(stations);
	Assert.assertEquals(stations.get(0).getId(), station.getId());
	
	// Remove station
	getService().removeStation(stationId);

	// Verify number of stations in DB after deletion
	final Integer sizeAfterRemove = getService().getNbStations();
	assertNotNull(sizeAfterRemove);
	Assert.assertEquals((Integer) sizeBeforeCreate, sizeAfterRemove);
    }

    @Test
    public void testSelectStation() throws Exception
    {
	// Create station1
	final Station station1 = createStation("stationTest1", STATION_ALTITUDE, STATION_LATITUDE, STATION_LONGITUDE);
	assertNotNull(station1.getId());

	// Create station2
	final Station station2 = createStation("stationTest2", STATION_ALTITUDE, STATION_LATITUDE, STATION_LONGITUDE);
	assertNotNull(station2.getId());

	// Create station3
	final Station station3 = new Station();
	station3.setName("stationTest3");
	station3.setAltitude("15");
	final Long stationId3 = getService().createStation(station3);
	assertNotNull(stationId3);

	// Verify number of stations in DB after creation
	final int size = getService().getNbStations();
	Assert.assertEquals(size, 3);

	// Select by altitude
	final List<Station> stationsByAltitude = getService().getStationsByAltitude(STATION_ALTITUDE);
	assertNotNull(stationsByAltitude);
	Assert.assertEquals(stationsByAltitude.size(), 2);
	Assert.assertEquals(stationsByAltitude.get(0).getId(), station1.getId());
	Assert.assertEquals(stationsByAltitude.get(1).getId(), station2.getId());
    }

    @Override
    protected void beforeServiceTest() throws Exception
    {
	afterServiceTest();
    }

    private static final String STATION_NAME = "stationNameTest";
    private static final String STATION_ALTITUDE = "10";
    private static final String STATION_LATITUDE = "20";
    private static final String STATION_LONGITUDE = "45";
}
